import java.util.Vector;

/**
 * Represents a binary tree where every father node is lower than its children
 * 
 * @author rcrespo
 * 
 */
public class BinaryHeap implements PriorityQueue {

  /**
   * Data structure that stores the information
   */
  Vector heap;

  public BinaryHeap() {
    heap = new Vector();
  }

  /**
   * Devuelve el n�mero de elementos Returns the number of elements
   */
  public int size() {
    return heap.size();
  }

  /**
   * Indicates if it is empty.
   */
  public boolean isEmpty() {
    return heap.isEmpty();
  }

  /**
   * Inserts the object in the right position, depending on its priority.
   * Priority is indicated by means of a natural number (> 0). The bigger is the
   * number, the lower is the priority of the associated element. Value 1
   * corresponds to the elements with highest priority.
   */
  public void insert(int priority, Object info) {

    PriorityQueueElement newE = new PriorityQueueElement(info, priority);

    int newEPos = heap.size();
    int fatherPos = (newEPos - 1) / 2;

    // Add a new element at the end of the heap
    heap.add(newE);
    PriorityQueueElement father = (PriorityQueueElement) heap.get(fatherPos);

    // Re-locate the new element to guarantee the order
    while (fatherPos >= 0 && newE.getPriority() < father.getPriority()) {

      // swapping elements
      heap.set(fatherPos, newE);
      heap.set(newEPos, father);

      // Calculate positions again
      newEPos = fatherPos;
      fatherPos = (newEPos - 1) / 2;
      father = (PriorityQueueElement) heap.get(fatherPos);
    }

  }

  /**
   * Extract the object with the highest priority (the one with priority with
   * minimum value) If heap is empty, returns null.
   */
  public Object extract() {

    Object info = null;
    boolean located = false;
    int holePos, child1Pos, child2Pos, minChildPos;

    if (!isEmpty()) {
      // Object to extract: the one on the root of the heap
      info = ((PriorityQueueElement) heap.get(0)).getInfo();

      // Extract the last element
      holePos = size() - 1;
      PriorityQueueElement hole = (PriorityQueueElement) heap.remove(holePos);

      // Rearrange the heap(if there are more elements)
      if (holePos > 0) {
        // The last element pass to the root
        holePos = 0;
        heap.set(holePos, hole);

        // Check the order while it has children
        child1Pos = holePos * 2 + 1;
        child2Pos = holePos * 2 + 2;
        while (!located && child1Pos < size()) {

          // Found the minimum of the 2 children, if any
          minChildPos = child1Pos;
          if ((child2Pos < size())
              && ((PriorityQueueElement) heap.get(child2Pos)).getPriority() < ((PriorityQueueElement) heap
                  .get(child1Pos)).getPriority()) {
            minChildPos = child2Pos;
          }

          // Compare the lower child with the new father
          if (hole.getPriority() > ((PriorityQueueElement) heap
              .get(minChildPos)).getPriority()) {

            heap.set(holePos, heap.get(minChildPos));
            heap.set(minChildPos, hole);

            // Calculate positions again
            holePos = minChildPos;
            child1Pos = holePos * 2 + 1;
            child2Pos = holePos * 2 + 2;
          } else {
            // It's in the correc position
            located = true;
          }
        }
      }
    }

    return info;
  }

  /**
   * Prints on the screen the elements stored and their priority, in the order
   * stated by the heap.
   */
  public void print() {
    for (int i = 0; i < heap.size(); i++) {
      System.out.println(((PriorityQueueElement) heap.get(i)).toString());
    }
  }

}
